/**********************************************************************
 *  fluxbg.cc   version 0.7.1
 *  License: GPL
 *  all stuff done by Juergen Repolusk <juerep@gmx.at>
 *
 *  FluxboxBackground:
 * =====================
 *  this programm is for setting up a nice background with
 *  a little graphical interface.
 *  i did it because i got bored from working with bsetbg command
 *
 *
 **********************************************************************/




#include <iostream>
#include <string>
#include <stdlib.h>
#include <gtkmm.h>
#include "fluxbg.hh"

extern "C"{
#include "getinfo.h"
#include "bgset.h"
#include "filefunc.h"
}

using namespace std;

const char * solid_temp_pic = "/tmp/fluxbg_tmp.pic"; // maybe add something like the pid()

FluxbgWindow::FluxbgWindow()
:
pic_center("Center"), pic_full("Full"), pic_tile("Tile"),
open_pic_button("Get Image"), import_list("Import List"),
export_list("Export List"),
aply_button("Aply"), okay_button("Okay"), cancel_button("Cancel"),
solid_button("Select Color"),
push_button("Add to list"), pull_button("pull")
{

	sel_full = true;
	sel_center = false;
	sel_tile = false;

	// main stuff & settings
	set_title("-=< Fluxbox Background >=-");
	set_border_width(2);
	set_default_size(600,400);
        set_resizable(false);

	// the box for the aply, okay and cancel button
	dobox.set_layout(Gtk::BUTTONBOX_END);
	dobox.add(aply_button);
	aply_button.show();

	dobox.add(okay_button);
	okay_button.show();

	dobox.add(cancel_button);
	cancel_button.show();
	// end of the box for aply, okay and cancel


	// here comes the radio buttons within the alignment frame
	alignment_frame.set_label("Alignment");
	// alignment.show();

	Gtk::RadioButton::Group group = pic_center.get_group();
	pic_full.set_group(group);
	pic_tile.set_group(group);

	pic_alignment.add(pic_full);
	pic_full.show();
	pic_alignment.add(pic_center);
	pic_center.show();
	pic_alignment.add(pic_tile);
	pic_tile.show();
	pic_full.set_active();

	//setbox.add(pic_alignment);
	alignment_frame.add(pic_alignment);
	pic_alignment.show();

	setbox.add(alignment_frame);
	alignment_frame.show();
	// end of radio buttons stuff

	select.set_layout(Gtk::BUTTONBOX_SPREAD);


	//select.add(pull_button); // not needed now!
	//pull_button.show();

	select.add(open_pic_button);  // open picture button
	open_pic_button.show();

	select.add(push_button); // add to list button
	push_button.show();

	select.add(export_list); // export to file button
 	export_list.show();

	//select.add(import_list); // not needed now!
	//import_list.show();

	//setbox.add(recent_used); // not needed now!
	//recent_used.show();

	setbox.add(select);
	select.show();

	//setbox.add(recent_used);
	//recent_used.show();


	// picture box with frame

	pic_frame.set_label("Preview");
	pic_frame.set_size_request(400,300); // does the fix sized frame
	pic_frame.add(pic_preview);
	pic_preview.show();

	// end of picture box


	color_buttons.add(solid_button);
	solid_button.show();

	color_box.add(color_buttons);
	color_buttons.show();
	color_box.add(solid_note);
	// solid_note.set_text("ATTENTION: because of the bsetbg command \nthe colorset is restricted to 0 to 9. That meens \nthat the color your see will not be 100% the\nbackground color. Just press the aply button\nto see how it really looks like");
	solid_note.set_text("ATTENTION: caused of the bsetbg/fbsetbg command \nit could be that the colorset function is not working.\nthis function will get better included in the 0.8 release.");
	solid_note.show();

	hbox2.add(color_box);
	color_box.show();

	col_frame.add(area_solid);
	sel_col.set_red(0);
	sel_col.set_blue(62533);
	sel_col.set_green(0);
	area_solid.modify_bg(Gtk::STATE_NORMAL, sel_col);
	area_solid.show();
	col_frame.set_label("Color Preview");
	//col_frame.show();

	hbox2.add(col_frame);
	col_frame.show();


	hbox3.add(about_story);
	about_story.set_text("fluxbg 0.7\nFluxbox Background Tool\n\nCopyright (c) 2003 by Juergen Repolusk\n<juerep@gmx.at>\n\nReleased under the GNU Public License");
	about_story.set_justify(Gtk::JUSTIFY_CENTER);
	about_story.show();


	// merging all boxes together

	hbox1.add(setbox);
	setbox.show();

	hbox1.add(pic_frame);
	pic_frame.show();

	main_tab.append_page(hbox1, "Picture");
	main_tab.append_page(hbox2, "Solid");
	main_tab.append_page(hbox3, "About");
	vbox1.add(main_tab);

	hbox3.show();
	hbox2.show();
	hbox1.show();

	main_tab.show();


	vbox1.add(dobox);
	dobox.show();


	// add the vbox inot the application window
	add(vbox1);
	vbox1.show();


	//add(main_tab);
	//main_tab.show();

	// signal connections
	okay_button.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::say_done));
	aply_button.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_aply));
	cancel_button.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::say_goodbye));

	open_pic_button.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::open_pic));
	import_list.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_import));
	export_list.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_export));

	// alignment radio buttons signals
	pic_center.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_center));
	pic_full.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_full));
	pic_tile.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_tile));

	solid_button.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_solid));
	// push & pull
	push_button.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_push));
	pull_button.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::do_pull));

	//connections for the tabs
	//main_tab.signal_clicked().connect(SigC::slot(*this, &FluxbgWindow::sel_solid_tab));

}

void FluxbgWindow::free_resources(void)
{
	free_bg_command();  // frees the get_bg_command() return value

}




void FluxbgWindow::do_push(void)
{
	
	// we use a class here because after use - the class is going to be destroyed and the memory released
	class temp_class{
	public:
		char *temp;
	};
	temp_class temper;
	
	temper.temp = (char *) malloc(strlen(sel_file) + 1); // reserves memory for char *
	
	strcpy( temper.temp, sel_file);
	// cout << temper.temp << endl;  // just testing

	test_vec.push_it(temper.temp, get_option());
	cout << "pushed file/option into file vector" << endl;
	
}

void FluxbgWindow::do_pull(void)
{
	try{
		//test_vec.print_it();
		test_vec.push_to_file("test.dat");
	} catch(...){
		cerr << "an pull failure has been occured!" << endl;
	}
}



char* FluxbgWindow::get_option(void)
{
	switch(main_tab.get_current_page())
	{
	case 0:
	{
		filename = sel_file;
		if (sel_full)
			return "-f";
		if (sel_tile)
			return "-t";
		if (sel_center)
			return "-c";
		break;
	}
	case 1:
	{
		//filename = sel_col;
		//TODO: if reloaded from picture then -f option should be set!!!!!
		
		//filename = solid_temp_pic;
		//return "-f";
		
		return "-solid"; // TODO: remove this line when temp-pic stuff is working
		break;
	}
	default:
		// should not be reched
		break;
	}

}


void FluxbgWindow::do_aply(void)
{
	option = get_option();

	if(set_bg()==0){
		// nothing to do here
		// cout << "background set..." << endl; // just testing
	} else {
	warning_dialog("Background could not be set!");
	
	};



}

void FluxbgWindow::say_done(void)
{
	// the same as before
	if(set_bg()==0){
		//cout << "background set..." << endl; // just testing
		//cout << "exit from programm!" << endl;
		Gtk::Main::quit();
	} else {
	warning_dialog("Background could not be set!");

		//nothing to do here
		//cout << "no background set..." << endl; // just testing
		//cout << "exit from programm!" << endl;
	};
	free_resources();
	Gtk::Main::quit();

}

void FluxbgWindow::do_import(void)
{
	char* local_path = get_user_path();
	Gtk::FileSelection select_import_file("import list from...");
	select_import_file.set_filename(local_path);
	select_import_file.run();
	
	free_get_user_path_res(); // frees the resourses from get_user_path()
	// here there should come the import list stuff
	warning_dialog("not implemented until now!");

}

void FluxbgWindow::do_export(void)
{
	const char * export_filename;
	char* local_path = get_user_path();
	Gtk::FileSelection select_export_file("export list to...");
	select_export_file.hide_fileop_buttons();
	select_export_file.set_filename(local_path);
	// build in a switch like in do_solid to enable a nice
	// cancel other button handling
	int result = select_export_file.run();
	switch(result)
	{
	case(Gtk::RESPONSE_OK) :
		{
		export_filename = select_export_file.get_filename().c_str();
		try{
			test_vec.push_to_file(export_filename);
		} catch (...) {
			cerr << "cannot push file-vector to file" << endl;
		}
		break;
		}
	case(Gtk::RESPONSE_CANCEL) :
		{
		// nothing to do here!
		break;
		}
	default:
		{
		// should not be reached
		break;
		}
	}


	free_get_user_path_res(); // frees the resourses from get_user_path()

}



void FluxbgWindow::open_pic(void)
{
	char* local_path = get_user_path();
	Gtk::FileSelection select_file("open image...");
	if (local_path != NULL)
		select_file.set_filename(local_path);
	// allow to see hidden directories
	select_file.hide_fileop_buttons();
	select_file.run();


	free_get_user_path_res(); // frees the resourses from get_user_path()

	filename = select_file.get_filename().c_str();
	sel_file = filename;


	Glib::RefPtr<Gdk::Pixbuf> pixbuf;
	Glib::RefPtr<Gdk::Pixbuf> pixbuf2;
	//Gtk::Image *image;
	pixbuf = Gdk::Pixbuf::create_from_file(filename);
	//image = manage( new Gtk::Image(pixbuf) );
	//pixbuf = image->get_pixbuf();
	pixbuf2 = pixbuf->scale_simple(400,300, Gdk::INTERP_BILINEAR);

	// the more complicate way:
	//pixbuf->Gdk::Pixbuf::scale(pixbuf2, 0,0,200,100,0,0,0,0,Gdk::INTERP_BILINEAR);


	pic_preview.set(pixbuf2);

	//pic_preview.set(filename);



	//cout << "opening: " << filename << endl; // just testing
}

void FluxbgWindow::say_goodbye(void)
{
	// cout << "lets get out of here!" << endl;
	free_resources();
	Gtk::Main::quit();
}

void FluxbgWindow::do_center(void)
{
	sel_full = false;
	sel_center = true;
	sel_tile = false;
	sel_solid = false;
	//cout << "center mode active..." << endl; // just for testing
}

void FluxbgWindow::do_full(void)
{
	sel_full = true;
	sel_center = false;
	sel_tile = false;
	sel_solid = false;
	//cout << "full mode active..." << endl; // just for testing
}

void FluxbgWindow::do_tile(void)
{
	sel_full = false;
	sel_center = false;
	sel_tile = true;
	sel_solid = false;
	//cout << "tile mode active..." << endl; // tile mode active
}



void FluxbgWindow::do_solid(void)
{
	// Gdk::Color sel_col;
	int red;
	int green;
	int blue;

	// sel_solid = true;
	option = get_option();

	Gtk::ColorSelectionDialog dialog;
	Gtk::ColorSelection* sel = dialog.get_colorsel();
	Glib::RefPtr<Gdk::Pixbuf> tempo_pix;

	// Glib::RefPtr<Gdk::Colormap> sel_col_map;

	

	int result = dialog.run();

	switch(result)
	{
		case(Gtk::RESPONSE_OK) :
		{
		  sel_col = sel->get_current_color();
	          area_solid.modify_bg(Gtk::STATE_NORMAL, sel_col);
		  
		  red = sel_col.get_red();
		  blue = sel_col.get_blue();
		  green = sel_col.get_green();

	// TODO: function to save as a picture in a /tmp directory
	
	/*
	Gdk::Pixbuf *pixbuf_tmp = NULL;
	Gdk::Pixmap *pixmap_tmp = NULL;
	Gdk::Drawable *drawable_tmp = NULL;

	area_solid_GC = Gdk::GC::create(area_solid.get_window());
	area_solid_cm = area_solid_GC->get_colormap();
	area_solid_cm->alloc_color(sel_col);
	
	area_solid_GC->set_background(sel_col);
	
	pixmap_tmp->create(area_solid.get_window(),20,20,8);
	
	pixbuf_tmp->create(pixmap_tmp->Drawable(), area_solid_cm, 0, 0, 1, 1, 10, 10);


	cout << "before" << endl; 
	cout << "before save" << endl;
	pixbuf_tmp->save("/tmp/fluxbgtst", "jpg");
	cout << "after save" << endl;

	*/


		  if (set_solid_bg(red, blue, green)!=0)
		  	// cout << "could not setup colors" << endl;

		  break;
		}
		case(Gtk::RESPONSE_CANCEL) :
		{
		    // cout << "Canceled Selection" << endl;
	 	  break;
		}
		default:
		{
		  // never reached!
		   cout << "unexpected button clicked" << endl;
		  break;
		}

	}

}


void FluxbgWindow::warning_dialog(char *message)
{
	Gtk::MessageDialog dialog(*this, message, Gtk::MESSAGE_WARNING, (Gtk::ButtonsType) (Gtk::BUTTONS_OK));
	dialog.run();
}

void FluxbgWindow::error_dialog(char *message)
{
	Gtk::MessageDialog dialog(*this, message, Gtk::MESSAGE_ERROR, (Gtk::ButtonsType) (Gtk::BUTTONS_OK));
	dialog.run();

}


gint FluxbgWindow::delete_event_impl(GdkEventAny*)
{
	Gtk::Main::quit();
	return true;
}




int main(int argc, char *argv[])
{
 Gtk::Main kit(argc, argv);

 FluxbgWindow window;
 window.show();

 Gtk::Main::run(window);

 return 0;

}
