/************************************************************
 *  fluxbg.hh   version 0.2
 *  License: GPL
 *  all stuff done by Juergen Repolusk <juerep@gmx.at>
 *
 *  see into fluxbg.cc for mor info
 *
 ***********************************************************/



#include <gtkmm.h>
#include <vector>
#include "file_vect.hh"

using namespace std;

class FluxbgWindow : public Gtk::Window
{
	/// Boxes
	Gtk::VBox vbox1, setbox, pic_alignment;
	//Gtk::VBox picbox;
	Gtk::HBox hbox1, hbox2, hbox3;
	//Gtk::VBox main_vbox2;
	Gtk::VBox superbox;
	Gtk::VBox color_box;

	/// Button Boxes
	Gtk::HButtonBox dobox;
	Gtk::VButtonBox select;
	Gtk::VButtonBox color_buttons;

	///Radio Buttons
	Gtk::RadioButton pic_center, pic_full, pic_tile;

	/// Buttons
	Gtk::Button open_pic_button, import_list, export_list;
	Gtk::Button aply_button, okay_button, cancel_button;
	Gtk::Button solid_button;
	Gtk::Button push_button, pull_button;

	/// Image and Drawing Stuff
	Gtk::Image pic_preview;
	Gtk::DrawingArea area_solid;
	Gdk::Color sel_col;

	/// Label & Frames
	Gtk::Label about_story, under_work, solid_note;
	Gtk::Frame pic_frame, alignment_frame;
	Gtk::Frame col_frame;

	/// Other stuff
	Gtk::TreeView recent_used;
	Gtk::Notebook main_tab;

protected:
	Glib::RefPtr<Gdk::GC> area_solid_GC;
	Glib::RefPtr<Gdk::Colormap> area_solid_cm;
	
	virtual gint delete_event_impl(GdkEventAny*);

	/// filename to open
	const char* sel_file;

	/// bsetbg options
	bool sel_tile;
	bool sel_center;
	bool sel_full;
	bool sel_solid;

	//vector<char *> file_vector;
	///file vector for pushing
	file_class test_vec;

public:
	FluxbgWindow();
	/// aplys current settings and exits the program
	void say_done(void);
	/// aplys the current settings
	void do_aply(void);
	/// quits the program
	void say_goodbye(void);
	/// this function is responsible for showing the open-dialog and the picture preview
	void open_pic(void);
	/// allowes to import a picture list
	void do_import(void);
	/// allowes to export a picture list
	void do_export(void);
	/// function to set the center alignment
	void do_center(void);
	/// function to set the full alignment
	void do_full(void);
	/// function to set the tile alignment
	void do_tile(void);
	/// function to activate the solid color mode
	void do_solid(void);
	/// gives out a warning
	void warning_dialog(char *);
	/// shows an error message
	void error_dialog(char *);
	/// responsible for pushing elements into a picture list
	void do_push(void);
	/// responsible for pulling elements out of a picture list
	void do_pull(void);
	/// function that returns the actually selected options
	char* get_option(void);
	/// function to free allocated resources
	void free_resources(void);

};

