/*************************************************************
 * fluxbgd.c version 0.5.2
 * License: GPL
 * all stuff done by juergen repolusk <juerep@gmx.at>
 *
 * fluxbox background daemon:
 *===========================
 *  ...is responsible for changing the background regualary
 *  if you want it. :) everything done in pure C.
 *
 *************************************************************/


 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <signal.h>

 #include "getinfo.h"
 #include "get_bg_command.h"


 /* may switch integer to unsigned for portability resons */
 int change_interval = 30; /* standard is every 30 minutes */
 int change_interval_sec;
 
 int rand_num;
 int rand_activate = 0;
 
 char * fluxbgd_ver = "0.5.2";
 char * fluxbg_list;  /*standard config file */
 
 const char *szCommand = "<not yet set>";
 FILE *fp;


 void Usage(void)
 {
	(void) fprintf(stderr, "Usage: %s [-i interval] [-f filename]\n", szCommand);
	exit(EXIT_FAILURE);

 }

 void PrintVersion(void)
 {
 	printf("Fluxbox background daemon (fluxbgd) version: %s\n", fluxbgd_ver);
	printf("(c) 2003 by Juergen Repolusk <juerep@gmx.at> \n");
 }


 void PrintHelp(void)
 {
	PrintVersion();
	printf("\n");
	printf("-i <interval>\t\tsets time interval for changing\n\t\t\tallowed from 0 to 546 minutes\n");
	printf("-f <filename>\t\tsets other filename for imports list\n\t\t\tstandard is the .fluxbgd file\n");
	printf("-r\t\t\tactivates randomizer\n");
	printf("-h\t\t\tthis message\n");
	printf("-v\t\t\toutput version information\n");
	exit(EXIT_SUCCESS);
 }


 void PrintError(const char*szMessage)
 {
 	if(errno != 0)
	{
		(void) fprintf(stderr, "%s: %s - %s\n", szCommand, szMessage, strerror(errno));
	}
	else
	{
		(void) fprintf(stderr, "%s: %s\n", szCommand, szMessage);
	}

 }

 void AllocateResources(void)
 {
	/* nothing to do until now */
 }

 void FreeResources(void)
 {
 	/* nothing to do until now */
	free_bg_command();
 	free(fluxbg_list);
 }


 void Handler(int nSignal)
 {
	FreeResources();
	exit(EXIT_SUCCESS);
 }



 int main(int argc, char **argv)
 {

 int c;
 int bOptionA = 0;
 int bError = 0;
 char * szInputFile = (char *) 0;
 char * szInterval = (char *) 0;
 char * eptr; /* seems not to be used */
 char * next_set[120];  /* maximum line size */

/* TODO: next_set could be fixed to any length !*/

int line_max ;
int current_line ;
int last_line ;
char *sys_command;
char * user_path;
char * fluxbg_list_name;

srand((unsigned) time(NULL)); // initialize new random

 szCommand = argv[0];

 while ((c = getopt(argc, argv, "vhri:f:")) != EOF)
 {
 	switch(c)
	{
		case 'v':
			PrintVersion();
			exit(EXIT_SUCCESS);
		break;

		case 'h':
			PrintHelp();
		break;


		case 'r':
			/*sets the randomizer function*/
			rand_activate = 1;
		break;


		case 'i':
		if (szInterval != (char *) 0)
		{
			bError = 1;
			break;
		}
		szInterval = optarg;
		/* convert the szInterval into an integer */
		change_interval = strtod(szInterval, NULL);

		if (change_interval <= 0)
		{
			PrintError("Interval zero or less");
			bError = 1;
			break;
		}
		break;

		case 'f':
		if(szInputFile != (char *) 0)
		{
			bError = 1;
			break;
		}
		szInputFile = optarg;
		break;

		case '?':
			bError = 1;
		break;


		default:
			/* nothing to do here*/
		break;
	}

 }

if (bError)
{
	Usage();
}
if (optind != argc)
{
	Usage();
}

/* Signal handlers*/
(void) signal (SIGTERM, Handler);
(void) signal (SIGINT, Handler);
(void) signal (SIGQUIT, Handler);


if (change_interval>564)
	{
	PrintError("Time interval too long");
	exit(EXIT_FAILURE);
	}
change_interval_sec = change_interval * 60; /* maybe check the size */


 printf("Starting up fluxbg daemon...\n");

 if (szInputFile != (char *) 0){
 	fluxbg_list = szInputFile;
 } else {
	/*now we are checking for the home directory of the user*/
	user_path = get_user_path();
	fluxbg_list_name = ".fluxbgl";
	/* merge user path and filename together */
	fluxbg_list = (char *) malloc(strlen(user_path)+strlen(fluxbg_list_name)+1); /*gets free at FreeResources()*/
	if (sprintf(fluxbg_list, "%s%s", user_path, fluxbg_list_name) < 0){
		PrintError("Could not merge user path with standard fluxbg filename together");
	};
 	free_get_user_path_res();
 
 }


 /* printf("...with data from %s\n",fluxbg_list); /* just testing */
 /* printf("and interval of %d minutes\n",change_interval); /* just testing */


	/* endless loop for deamon */
	while(1)
	{

	/*opening file */
	if ((fp = fopen(fluxbg_list,"r"))==NULL)
		{
		PrintError("Could not open file");
		exit(EXIT_FAILURE);
		}
	line_max = 0;
	current_line = 0;
	last_line = 0;
	/* next while loop is checking how much lines are in the file */

	while (fgets (next_set, sizeof(next_set), fp) != NULL)
	{
		line_max++;
	}
	fclose(fp);

	/* printf("got %d lines\n", line_max); /* just testing  */


	if (rand_activate == 1)
	{
		rand_num = ((float) rand()/RAND_MAX)*line_max;
		/* printf("random number is: %d \n", rand_num); /* just testing */
		last_line = rand_num;
	}



	while(1)
	{
	current_line = 0;

	if (last_line == line_max){
		last_line = 0;
	}


	if ((fp = fopen(fluxbg_list,"r"))==NULL)
		{
		PrintError("Could not open file");
		exit(EXIT_FAILURE);
		}

	/* next while should set the current line to the point of the last line */
	while(current_line != last_line)
	{
		/* just do nothing here until actually line has reached*/
		if (fgets(next_set, sizeof(next_set), fp)==NULL){
			PrintError("Cannot read out line from current file!");
			exit(EXIT_FAILURE);
		};
		current_line++;
	}

	/* printf("current line is %d \n", current_line); /* just testing*/

	last_line++;

	/* read out the next set that should be displayed*/
	if (fgets(next_set, sizeof(next_set), fp)==NULL){
			PrintError("Cannot read out line from current file!");
			exit(EXIT_FAILURE);
		};

	/* merge it together with the BG_COMMAND */
	sys_command = (char *) malloc(strlen(get_bg_command())+1+strlen(next_set)+1);
	if (sprintf(sys_command, "%s %s", get_bg_command(), next_set) < 0){	
	PrintError("Could not merge command with options together");
		};
	/* set the background now */
	if (system(sys_command)==-1) {
			PrintError("Could not execute command from file");
			};

	free(sys_command); /*free allocated ressources for sys_command*/

	fclose(fp);
	/* file is closed now and the daemon ready to sleep */
	sleep(change_interval_sec);

	}	/* 2nd while(1) ends here*/


	}	/* main while(1) loop ends here */

 return 0; /* never reached! */

 }

